==> wrogue-0.7.4.tar.bz2: wrogue-0.7.4/src/areatran.c <==
/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Module Name: AreaTransition
 * Description: -
 */

#define Uses_Ui
#define Uses_World
#define Uses_Area
#define Uses_Sector
#define Uses_Character
#define Uses_Event
#define Uses_LoadSave
#define Uses_ProgramManager
#define Uses_Death
#define Uses_WorldGenerator
#include "mheader.h"

#include "areatran.h"


static LIST *		CharacterTransferList = NULL;
static LIST *		EventTransferList = NULL;


static void		grab_characters_to_transfer(void);

static void		transfer_characters(const CHARACTER *, DIRECTION);
static AREA_POINT *	location_after_transition(AREA_POINT *, const CHARACTER *, DIRECTION);



/*
 * moves the player's party to a different area
 */
void move_party_to_area(const CHARACTER *leader, 
	const WORLD_POINT *target_area,
	DIRECTION move_direction
)
{

	if (out_of_world_bounds(target_area) ||
		world_points_equal(target_area,
			&active_area()->coord)) {

		return;
	}

	save_point_create();

	remove_splatter();

	respawn_npcs();

	grab_characters_to_transfer();

	area_write(NULL);

	area_destroy();

	area_read(target_area);

	transfer_characters(leader, move_direction);

	update_view_character(player_controlled_character());
}



/*
 * grabs all characters which are supposed to be transfered
 */
static void grab_characters_to_transfer(void)
{
	LIST_NODE *node;

	CharacterTransferList = list_new();
	EventTransferList = list_new();

	for (node = area_character_list()->head;
		node != NULL;
		) {
		CHARACTER *character;

		character = (CHARACTER *)node->data;
		node = node->next;

		if (character->party != PARTY_PLAYER) {

			continue;
		}

		remove_character(character, EventTransferList);

		list_add(CharacterTransferList, character);
 	}
}



/*
 * transfers characters
 */
static void transfer_characters(const CHARACTER *leader, DIRECTION move_direction)
{
	LIST_NODE *node;
	AREA_POINT spawn_start;

	location_after_transition(&spawn_start, leader, move_direction);

	for (node = CharacterTransferList->head;
		node != NULL;
		node = node->next) {
		CHARACTER *character;
		AREA_POINT area_point;

		character = (CHARACTER *)node->data;

		area_point = spawn_start;

		if (!sector_is_class(&area_point, SC_MOVE_TARGET_SAFE)) {
			SECTOR *sector;

			sector = adjacent_sector(&area_point,
				SC_MOVE_TARGET_SAFE
			);

			if (sector == NULL) {

				die("*** CORE ERROR *** no room "\
					"to transfer characters"
				);
			}
		}

		place_character(character, &area_point);
	}

	list_destroy(CharacterTransferList);

	list_add_list(area_event_list(), EventTransferList);
	list_destroy(EventTransferList);
}



/*
 * returns the location at which the party starts after transition
 */
static AREA_POINT * location_after_transition(AREA_POINT *location, const CHARACTER *leader, DIRECTION move_direction)
{

	if (move_direction == DIRECTION_NIL) {

		return area_elevator_location(location);

	} else if (move_direction == DIRECTION_LEFT) {

		location->x = AREA_WIDTH - 1;
		location->y = leader->location.y;

	} else if (move_direction == DIRECTION_RIGHT) {

		location->x = 0;
		location->y = leader->location.y;

	} else if (move_direction == DIRECTION_UP) {

		location->x = leader->location.x;
		location->y = AREA_HEIGHT - 1;

	} else if (move_direction == DIRECTION_DOWN) {

		location->x = leader->location.x;
		location->y = 0;

	} else if (move_direction == DIRECTION_UP_LEFT) {

		location->x = AREA_WIDTH - 1;
		location->y = AREA_HEIGHT - 1;

	} else if (move_direction == DIRECTION_UP_RIGHT) {

		location->x = 0;
		location->y = AREA_HEIGHT - 1;

	} else if (move_direction == DIRECTION_DOWN_LEFT) {

		location->x = AREA_WIDTH - 1;
		location->y = 0;

	} else if (move_direction == DIRECTION_DOWN_RIGHT) {

		location->x = 0;
		location->y = 0;
	}

	return location;
}

==> wrogue-0.7.5.tar.bz2: wrogue-0.7.5/src/areatran.c <==
/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Module Name: AreaTransition
 * Description: -
 */

#define Uses_Ui
#define Uses_World
#define Uses_Area
#define Uses_Sector
#define Uses_Character
#define Uses_Event
#define Uses_LoadSave
#define Uses_ProgramManager
#define Uses_Death
#define Uses_Dungeon
#include "mheader.h"

#include "areatran.h"


static LIST *		CharacterTransferList = NULL;
static LIST *		EventTransferList = NULL;


static void		grab_characters_to_transfer(void);

static void		transfer_characters(DIRECTION);

static AREA_POINT *	location_after_transition(AREA_POINT *, DIRECTION);




/*
 * moves the player's party to a different area
 */
void move_party_to_area(const WORLD_POINT *target_point,
	DIRECTION direction
)
{

	remove_splatter();

	respawn_npcs();

	grab_characters_to_transfer();

	area_write(NULL);

	area_destroy();

	area_read(target_point);

	transfer_characters(direction);

	update_view_character(player_controlled_character());
}



/*
 * grabs all characters which are supposed to be transfered
 */
static void grab_characters_to_transfer(void)
{
	LIST_NODE *node;

	CharacterTransferList = list_new();
	EventTransferList = list_new();

	for (node = area_character_list()->head;
		node != NULL;
		) {
		CHARACTER *character;

		character = (CHARACTER *)node->data;
		node = node->next;

		if (character->party != PARTY_PLAYER) {

			continue;
		}

		remove_character(character, EventTransferList);

		list_add(CharacterTransferList, character);
 	}
}



/*
 * transfers characters
 */
static void transfer_characters(DIRECTION move_direction)
{
	LIST_NODE *node;
	AREA_POINT spawn_start;

	location_after_transition(&spawn_start, move_direction);

	for (node = CharacterTransferList->head;
		node != NULL;
		node = node->next) {
		CHARACTER *character;
		AREA_POINT area_point;

		character = (CHARACTER *)node->data;

		area_point = spawn_start;

		if (!sector_is_class(&area_point, SC_MOVE_TARGET_SAFE)) {
			SECTOR *sector;

			sector = sector_adjacent(&area_point,
				SC_MOVE_TARGET_SAFE
			);

			if (sector == NULL) {

				die("*** CORE ERROR *** no room "\
					"to transfer characters"
				);
			}
		}

		place_character(character, &area_point);
	}

	list_destroy(CharacterTransferList);

	list_add_list(area_event_list(), EventTransferList);
	list_destroy(EventTransferList);
}



/*
 * returns the location at which the party starts after transition
 */
static AREA_POINT * location_after_transition(AREA_POINT *location,
	DIRECTION move_direction
)
{
	const AREA_SECTION *bounds;
	
	bounds = area_bounds();

	if (move_direction == DIRECTION_NIL) {

		sector_elevator(location);

		return location;

	} else if (move_direction == DIRECTION_LEFT) {

		location->x = bounds->right;
		location->y = bounds->bottom / 2;

	} else if (move_direction == DIRECTION_RIGHT) {

		location->x = bounds->left;
		location->y = bounds->bottom / 2;

	} else if (move_direction == DIRECTION_UP) {

		location->x = bounds->right / 2;
		location->y = bounds->bottom;

	} else if (move_direction == DIRECTION_DOWN) {

		location->x = bounds->right / 2;
		location->y = bounds->top;

	} else if (move_direction == DIRECTION_UP_LEFT) {

		location->x = bounds->right;
		location->y = bounds->bottom;

	} else if (move_direction == DIRECTION_UP_RIGHT) {

		location->x = bounds->left;
		location->y = bounds->bottom;

	} else if (move_direction == DIRECTION_DOWN_LEFT) {

		location->x = bounds->right;
		location->y = bounds->top;

	} else if (move_direction == DIRECTION_DOWN_RIGHT) {

		location->x = bounds->left;
		location->y = bounds->top;
	}

	return location;
}
